#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#define bit(x) (1 << (x))

#define RULE	30 // 22 // 
#define LEN	8
#define STEPS	64	// default steps

typedef	unsigned char uint8_t;
typedef	unsigned long long uint64_t;

//			   *    *    * ****   **  * **  *  *  ***** *** *   **   ***  ** 
//			 001000010000101111000110010110010010011111011101000110001110011	
// uint8_t uni[ LEN ] = { 0b00100001, 0b00001011, 0b11000110, 0b01011001, 0b00100111, 0b11011101, 0b00011000, 0b11100110 };

uint8_t uni[ LEN ] = { 0b00011100, 0b00010011, 0b01010001, 0b11011110, 0b11100001, 0b01111000, 0b01110011, 0b10010110 };

void shift_left( uint8_t shift) {

	while (shift--) {

		uint8_t carry = 0;
		uint8_t i = LEN - 1;

		do {
			uint8_t next = (uni[i] & 0x80) ? 1 : 0;
			uni[i] = carry | (uni[i] << 1);
			carry = next;

		} while ( i-- > 0 );

		uni[ LEN - 1 ] |= carry ;
	}
}  


int main( int argc, char *argv[] ){

	uint64_t	j = STEPS;
	uint8_t 	i, tmp, triade;
	uint8_t 	cnt=0, cntMax=0;
	uint8_t 	stat[ 8*LEN ];

	if ( argc < 2 ) {
		printf("CyberHeart evolution statistical analyzer. (◎)2021 by Dmitry Shalnoff, MIT\n");
		printf("Usage: %s [number of steps] [p]\n", argv[0]);
		printf("p:\tprint output state instead of statistics\n");
		return 0;
	} 

	j = atoi( argv[1] );

	// ------------ cell automata -------------

	if ( argc == 2 ){
		printf("Steps: %llu, about %llu min (>%llu realtime days with 8 sec epoch delay) \n", j, (j*8)/(60), (j*8)/(60*60*24) );
	}

	while( j-- ) {

		tmp = uni[ 0 ];

		if ( argc > 2  && argv[2][0] == 'p' ){
			for (i = 0; i < LEN*8; i++) putchar( uni[ i/8 ] & bit( 7-(i%8) ) ? '*' : ' ');
//			for (i = 0; i < LEN*8; i++) printf("%s", uni[ i/8 ] & bit( 7-(i%8) ) ? "▒▒" : "  ");
			putchar('\n');
		}

		for (i=0; i < LEN*8; i++){

//			if ( argv[2] == 'p' ){
//				put_char( uni[ i/8 ] & bit( 7-(i%8) ) );
//				putchar( uni[ i/8 ] & bit( 7-(i%8) ) ? '*' : ' ');
//			}

			if ( i > LEN * 8 - 4 ) triade = (tmp >> (LEN*8 - i) ); else triade = uni[ 0 ];
			if ( RULE & bit(7 & triade) ) uni[ 0 ] |= bit(2); else uni[ 0 ] &= ~bit(2);

			if ( (( uni[ 0 ] >> 2) & 0b11) == 0b00 ) { 

				cnt++;

				if ( cnt > cntMax ) {
					cntMax = cnt;
				}
			
			} else {
				stat[cnt]++;
				cnt = 0;
			}
			
			shift_left( 1 );
		}

		shift_left( LEN*8-1 );
	}

	if ( argc == 2){
		printf("cnt max: %d\n", cntMax );
		printf("stat: \n");
		for (i=0; i<=cntMax; i++) printf("%d: \t%d\n", i, stat[i]);
	}

}
